/*******************************************************************************
  PC Command Line Tool for use with serial PIC programmer

  Version 0.0			(C) 2016 Thomas Elger (thomas@picalic.de)
  
  This program is free software; you can redistribute it and/or modify it 
  under the terms of the GNU General Public License as published by the 
  Free Software Foundation; either version 3 of the License, or (at your 
  option) any later version.
  This program is distributed in the hope that it will be useful, but 
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
  for more details.

  For a copy of GNU General Public License, see <http://www.gnu.org/licenses/>. 
/*******************************************************************************/



#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <windows.h>
#include <time.h>
#include <conio.h>

#include "Options.h"
#include "Comport.h"
#include "hexread.h"
#include "SPRGcom.h"
#include "target.h"
#include "memwrite.h"

#define MAJ_VERSION 0			// program release version
#define MIN_VERSION 0

extern FILE *logfile;
extern int minAddr, maxAddr;
extern FLASH save_osccal, save_config;

void exitfunction(void)
{
	ComWrite(CMD_PRGOFF);
	ComWrite(0x100-CMD_PRGOFF);
	Sleep(50);
	ComClose();
	if(logflag)
	{
		fprintf(logfile,"\n--------------- end of log --------------\n");
		fclose(logfile);
	}
}

// *****************************************************************************
// 					m a i n 
// *****************************************************************************
int main(int argc, char** argv) 
{
	FILE *inputfile = NULL;
	char *inputfilename;
	FLASH flashdata;
	int i, temp, pstat;
	char ch;
	
	atexit(exitfunction);

	progflag = 0;
	save_osccal = 0x8000;					// bit 15: "empty"
	save_config = 0x8000;		

	
	for(i=0;i<MAXFLASHSIZE;i++) picflash[i] = 0xFFFF;		// initialize flash data to "empty"
	
	strcpy(picdata.picname,"(unknown)");

	inputfilename = processOptions(argc, argv);		// read config from file and set options from command line 

	if(infoflag)
		printf("%s Version %d.%d\n",progname,MAJ_VERSION,MIN_VERSION);
	
	if((pstat = GetTargetInfo()) < 0)
	{
		printf("\nno target detected!\n");
		exit(EXIT_FAILURE);
	}

	//-------------------------------------------------------------
	// OSCCAL calibration helper:
	// assumes that application toggles ICSPCLK every 65536 s
	
	if(osccalflag!=0)
	// void CalibrationHelper(void)
	{
		char c;
		int timestamp = clock();
		float sec_host = 0;
		float sec_target = 0;
	
		send2prg(CMD_RUN);
		flushTxBuffer();
		while(!kbhit())
		{
			if(ComGetReadCount() != 0)
			{
				c = ComRead();				// bootloader sends data byte for each toggle
				sec_target += 0.065536;
				sec_host = (float)(clock() - timestamp)/CLOCKS_PER_SEC;
				printf("\n%f",sec_target-sec_host);
//				printf("\n%f",sec_target);
			}
		}
		sendRdyAck();
	}
	
	//-------------------------------------------------------------
	// option RD: Read data from target
	if(read_from >= 0)
	{
		if (read_to < 0)
			read_to = read_from;
		if (read_to < read_from)
			read_to = MAXFLASHSIZE-1;
		if (read_to >= MAXFLASHSIZE)
			read_to = MAXFLASHSIZE-1;
			
		for(i=0;read_from<=read_to; read_from++)
		{
			temp = validMem(read_from);
			if(temp != 0) 									// skip non-existing memory areas
			{
				if((read_from & (temp==1?0x0F:0x07)) == 0)	// force newline at 8 or 16 words boundary
					i=0;									// (depending on section's word size)
				if(i==0)
					printf("\n  %04x: ", read_from);		// start a new line with address
				flashdata = readWord (read_from);
				if(temp == 1)								// range is data flash: size = 8 bits
					printf(" %02x", flashdata & 0xFF); 
				else 										// else words are 14 bits 
					printf(" %04x", flashdata); 
				i++;
			}
		}
	}

	//-------------------------------------------------------------
	// option ERASE: fill specified memory range with 0x3FFF 
	if((erase_from >= 0) && ((progflag & PRG_ERA) == 0))
	{
		saveCutRows(erase_from, erase_to);	// save part. overwritten memory, set progflag according to memory range
									
		for(i=erase_from;i<=erase_to;i++) 
		{
			if(validMem(i)>0)
				picflash[i] = 0x3FFF;		// MSB = 0 -> do programming here! 
		}
	}
	//-------------------------------------------------------------
	// option FILL: fill specified memory range with given number
	if(fill_from >= 0)
	{	
		saveCutRows(fill_from, fill_to);	// save part. overwritten memory, set progflag according to memory range
					
		for(i=fill_from;i<=fill_to;i++) 
		{
			if(validMem(i)>0)
			{
				picflash[i] = fill_data;
					
				if (T12F629 == picdata.type)
				{
					if(i == picdata.osccal_loc) 
						save_osccal = fill_data;			// allow manually overwriting calib data
					if(i == 0x8007)
						save_config = fill_data;			// (same for config)
				}
			}
		}
	}

	if (infoflag)
	{
		printf("\nTarget device ID:      0x%04X = %s", picdata.hardware_id, &picdata.picname);
		printf("\nProgram flash size:    0x%04X", picdata.flashsize);
		if(picdata.eeaddr)
			printf("\nData EEPROM:           %d", picdata.ee_size);
//		printf("\nFlash erase block:     0x%04X", picdata.rowsize);
		printf("\nTarget's user ID:      %04X", picdata.user_id);
		if(pstat & PSTAT_TRGVCC)
			printf("\nVDD by Target (CONFIG: MCLRE_OFF might cause problem!)\n");
		else
			printf("\nVDD by programmer\n");
	}

	//-------------------------------------------------------------
	// option hex file: read hex file 
	if(inputfilename != NULL)
	{
		inputfile = fopen(inputfilename, "r"); 
		if(inputfile == NULL)
		{
			printf("\ncannot open hex input file!\n");
			exit(EXIT_FAILURE);
		}
		else
		{
			temp = readHexFile(inputfile,picflash,picdata.flashsize);
			fclose(inputfile);			
			if (temp < 0)
				exit(EXIT_FAILURE);
			progflag |= temp;							// denotes which memory was written

		}
	}
	
	//---------------------- program the PIC
	programming();	

	return 0;
}
